-- Created by hey @ Widow / icheat@lan
-- Licensed under the GNU General Public License Version 3.  See http://www.gnu.org/licenses/gpl-3.0.txt for details.

local DyePreview = {}
DyePreview.__index = DyePreview

local form_doc
local form
local options_form

local dyes

local all_buttons
local slot_buttons

local current_dye_index = 1
local current_dye
local dye_entry_to_index = {}
local shown_dyes = {}

local options = {
  ["ShowUnknown"] = false
}

local preview_slots = {
  GameLib.CodeEnumItemSlots.Head,   GameLib.CodeEnumItemSlots.Shoulder, GameLib.CodeEnumItemSlots.Chest,
  GameLib.CodeEnumItemSlots.Hands,  GameLib.CodeEnumItemSlots.Legs,     GameLib.CodeEnumItemSlots.Feet
}

function DyePreview.new()
  local self = setmetatable({}, DyePreview)

  return self
end

function DyePreview:Init()
  Apollo.RegisterAddon(self)
end

function DyePreview:OnLoad()
  Apollo.RegisterSlashCommand("dp", "OnSlashCommand", self)
  Apollo.RegisterSlashCommand("dyepreview", "OnSlashCommand", self)

  form_doc = XmlDoc.CreateFromFile("DyePreview.xml")
  form = Apollo.LoadForm(form_doc, "DyePreviewWindow", nil, self)

  options_form = Apollo.LoadForm(form_doc, "OptionsWindow", form, self)

  local lang = "EN"
  local l_test = Apollo.GetString(1)
  if l_test == "Abbrechen" then
    lang = "DE"
  elseif l_test == "Annuler" then
    lang = "FR"
  end

  -- thanks Sinaloit
  dyes = dofile(string.match(Apollo.GetAssetFolder(), "(.-)[\\/][Aa][Dd][Dd][Oo][Nn][Ss]") .. "\\Addons\\DyePreview\\Dyes-" .. lang .. ".lua")
end

function DyePreview:OnSave(level)
  if level ~= GameLib.CodeEnumAddonSaveLevel.Account then
    return nil
  end

  return options
end

function DyePreview:OnRestore(level, save)
  if level ~= GameLib.CodeEnumAddonSaveLevel.Account then
    return nil
  end

  options = save
end

function DyePreview:OnClose(_, _, _)
  form:Show(false)  
end

function DyePreview:OnShowOptionsWindow(_, _, _)
  options_form:Show(true)
  options_form:FindChild("OShowUnknown"):SetCheck(options.ShowUnknown)
end

function DyePreview:OnCloseOptionsWindow(_, _, _)
  options_form:Show(false)
end

function DyePreview:OnToggleShowUnknown(_, control, _)
  options.ShowUnknown = control:IsChecked()

  self:BuildDyeList(form:FindChild("DyeContainer:DyeSelectorWindow:FilterKnownDyes"):IsChecked())
end

function DyePreview:OnToggleShowKnown(_, control, _)
  self:BuildDyeList(control:IsChecked())
end

function DyePreview:BuildDyeList(only_known)
  local dye_list = form:FindChild("DyeContainer:DyeSelectorWindow:DyeSearchList")
  dye_list:DestroyChildren()
  dye_entry_to_index = {}

  local known_dyes = {}
  for _, dye in ipairs(GameLib.GetKnownDyes()) do
    known_dyes[dye.nId] = true
  end

  for idx, dye in ipairs(dyes) do
    if dye[4] ~= "(Unnamed)" or options.ShowUnknown then
      if not only_known or known_dyes[dye[1]] then
        local entry = Apollo.LoadForm(form_doc, "DyeListEntry", dye_list, self)
        entry:FindChild("DyeName"):SetText(dye[4])
        entry:FindChild("SwatchContainer:DyeSwatch"):SetSprite("CRB_DyeRampSprites:sprDyeRamp_" .. dye[2])
        entry:SetData(dye)

        dye_entry_to_index[dye[1]] = idx

        if idx == 2 then -- XXX: temporary
          current_dye = dye
          current_dye_index = 2
          entry:SetCheck(true)
          form:FindChild("DyeContainer:DyeSelection"):SetText(entry:FindChild("DyeName"):GetText())
        end
      end
    end
  end

  dye_list:ArrangeChildrenVert()
end

function DyePreview:OnSlashCommand()
  form:Show(not form:IsVisible())

  if form:IsVisible() then
    local unit = GameLib.GetPlayerUnit()

    form:FindChild("CostumePreview"):SetCostume(unit)

    if not slot_buttons then
      self:AggregateButtons()
      self:BuildDyeList()
    end

    self:UpdateChannelAvailability()
  end
end

function DyePreview:GetPreviewItems()
  local costume = GameLib.GetCostumeIndex()
  local equipped = {}
  local items = {}
  local visibility = {}

   for _, item in ipairs(GameLib.GetPlayerUnit():GetEquippedItems()) do
    local slot = item:GetSlot() + 1
    equipped[slot] = item

    if costume == 0 then
      items[slot] = item
      visibility[slot] = not not item
    end
  end

  if costume > 0 then
    for _, slot in ipairs(preview_slots) do
      local item = GameLib.GetCostumeItem(costume, slot)
      visibility[slot] = GameLib.IsCostumeSlotVisible(costume, slot)

      if not item and visibility[slot] then
        item = equipped[slot]
      end

      items[slot] = item
    end
  end

  return items, visibility
end

function DyePreview:UpdateCostumeDye()
  local items, visibility = self:GetPreviewItems()

  local wnd = form:FindChild("CostumePreview")
  for slot, buttons in pairs(slot_buttons) do
    local shown = shown_dyes[slot]
    for i = 1, 3 do
      shown[i] = (buttons[i]:IsEnabled() and buttons[i]:IsChecked()) and current_dye[1] or shown_dyes[slot][i]
    end

    if items[slot] and visibility[slot] then
      wnd:SetItemDye(items[slot], shown[1], shown[2], shown[3])
    end
  end
end

function DyePreview:AggregateButtons()
  all_buttons = {}
  slot_buttons = {}

  local container = form:FindChild("DyeContainer")

  local all_slot_buttons = container:FindChild("AllSlotsContainer"):GetChildren()
  for i = 1, 3 do
    all_buttons[i] = all_slot_buttons[i]
    all_buttons[i]:SetData(i)
  end

  for _, slot_idx in ipairs(preview_slots) do
    local slot = container:FindChild("CostumeSlot" .. tostring(slot_idx))

    slot:SetData(slot_idx)

    slot_buttons[slot_idx] = {
      slot:FindChild("Color1"),
      slot:FindChild("Color2"),
      slot:FindChild("Color3")
    }

    shown_dyes[slot_idx] = { 0, 0, 0 }

    for i = 1, 3 do
      slot_buttons[slot_idx][i]:SetData(i)
    end
  end
end

function DyePreview:UpdateChannelAvailability()
  local items, visibility = self:GetPreviewItems()

  for slot, buttons in pairs(slot_buttons) do
    if items[slot] and visibility[slot] then
      local channels = items[slot]:GetAvailableDyeChannel()
      buttons[1]:Enable(channels.bDyeChannel1)
      buttons[2]:Enable(channels.bDyeChannel2)
      buttons[3]:Enable(channels.bDyeChannel3)
    else
      for i = 1, 3 do
        buttons[i]:Enable(false)
      end
    end
  end
end

function DyePreview:FilterDyeList(term)
  local list = form:FindChild("DyeContainer:DyeSelectorWindow:DyeSearchList")
  for _, entry in ipairs(list:GetChildren()) do
    local matches = not not string.find(string.lower(entry:FindChild("DyeName"):GetText()), string.lower(term), 1, true)
    entry:Show(matches)
    entry:Enable(matches)
  end

  list:ArrangeChildrenVert()
end

function DyePreview:OnDyeListEntry(_, control, _)
  form:FindChild("DyeContainer:DyeSelection"):SetText(control:FindChild("DyeName"):GetText())

  current_dye = control:GetData()
  current_dye_index = dye_entry_to_index[control:GetData()[1]]

  self:UpdateCostumeDye()
end

function DyePreview:OnDyeSelectionToggle(_, control, _)
  control:GetParent():FindChild("DyeSelectorWindow"):Show(control:IsChecked())
end

function DyePreview:OnColorChannelToggle(_, control, _)
  local slot_idx = control:GetParent():GetData()

  all_buttons[control:GetData()]:SetCheck(false)

  self:UpdateCostumeDye()
end

function DyePreview:OnPreviousDye(_, _, _)
  local dye_entries = form:FindChild("DyeContainer:DyeSelectorWindow:DyeSearchList"):GetChildren()

  local next_index = current_dye_index - 1
  while next_index > 0 and not dye_entries[next_index]:IsEnabled() do
    next_index = next_index - 1
  end

  if next_index ~= 0 then
    current_dye_index = next_index

    for idx, button in ipairs(dye_entries) do
      button:SetCheck(idx == current_dye_index)

      if idx == current_dye_index then
        current_dye = button:GetData()
        form:FindChild("DyeContainer:DyeSelection"):SetText(button:FindChild("DyeName"):GetText())
      end
    end

    self:UpdateCostumeDye()
  end
end

function DyePreview:OnNextDye(_, _, _)
  local dye_entries = form:FindChild("DyeContainer:DyeSelectorWindow:DyeSearchList"):GetChildren()

  local next_index = current_dye_index + 1
  while next_index <= #dye_entries and not dye_entries[next_index]:IsEnabled() do
    next_index = next_index + 1
  end

  if next_index <= #dye_entries then
    current_dye_index = next_index

    for idx, button in ipairs(dye_entries) do
      button:SetCheck(idx == current_dye_index)

      if idx == current_dye_index then
        current_dye = button:GetData()
        form:FindChild("DyeContainer:DyeSelection"):SetText(button:FindChild("DyeName"):GetText())
      end
    end

    self:UpdateCostumeDye()
  end
end

function DyePreview:ClearButtons()
  for _, b in ipairs(all_buttons) do
    b:SetCheck(false)
  end

  for _, buttons in ipairs(slot_buttons) do
    for _, b in ipairs(buttons) do
      b:SetCheck(false)
    end
  end
end

function DyePreview:OnApply(_, _, _)
  self:ClearButtons()
end

function DyePreview:OnReset(_, _, _)
  self:ClearButtons()

  for idx, _ in ipairs(shown_dyes) do
    shown_dyes[idx] = { 0, 0, 0 }
  end

  self:UpdateCostumeDye()
end

function DyePreview:OnAllToggle(_, control, _)
  for slot_idx, buttons in pairs(slot_buttons) do
    buttons[control:GetData()]:SetCheck(control:IsChecked())
  end

  self:UpdateCostumeDye()
end


function DyePreview:OnClearSearchTerm(_, control, _)
  local box = control:GetParent():FindChild("DyeSearchBox")
  box:SetText("")
  box:ClearFocus()

  self:FilterDyeList("")
end

function DyePreview:OnSearchTermChanged(_, control, _)
  self:FilterDyeList(control:GetText())
end

function DyePreview:OnRotateLeft(_, _, _)
  form:FindChild("CostumePreview"):ToggleLeftSpin(true)
end

function DyePreview:OnEndRotateLeft(_, _, _)
  form:FindChild("CostumePreview"):ToggleLeftSpin(false)
end

function DyePreview:OnRotateRight(_, _, _)
  form:FindChild("CostumePreview"):ToggleRightSpin(true)
end

function DyePreview:OnEndRotateRight(_, _, _)
  form:FindChild("CostumePreview"):ToggleRightSpin(false)
end

function DyePreview:OnToggleSheath(_, control, _)
  form:FindChild("CostumePreview"):SetSheathed(control:IsChecked())
end

DyePreview.new():Init()
